import * as __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__ from "react/jsx-runtime";
import * as __WEBPACK_EXTERNAL_MODULE_react__ from "react";
import * as __WEBPACK_EXTERNAL_MODULE__gant_lowcode_code_generator_standalone_loader__ from "@gant-lowcode/code-generator/standalone-loader";
import * as __WEBPACK_EXTERNAL_MODULE__gant_lowcode_lowcode_types__ from "@gant-lowcode/lowcode-types";
import * as __WEBPACK_EXTERNAL_MODULE_antd__ from "antd";
import * as __WEBPACK_EXTERNAL_MODULE_semver_functions_coerce_js__ from "semver/functions/coerce.js";
import * as __WEBPACK_EXTERNAL_MODULE__code_gen_result_index_js__ from "../code-gen-result/index.js";
import "./index.css";
import * as __WEBPACK_EXTERNAL_MODULE__ant_design_icons__ from "@ant-design/icons";
function CodeGenActionBtn({ ctx }) {
    const [state, setState] = (0, __WEBPACK_EXTERNAL_MODULE_react__.useState)({
        open: false,
        hasError: false,
        error: null,
        loading: false,
        result: null,
        schema: null,
        originalSchema: null
    });
    const handleClick = async ()=>{
        try {
            // 打开基于 Gravity 的编辑器/出码预览器
            setState((prev)=>({
                    ...prev,
                    loading: true,
                    open: true,
                    hasError: false
                }));
            console.log('--ctx', ctx);
            const getProjectSchema = ctx.config.get('getProjectSchema');
            let originalSchema;
            originalSchema = getProjectSchema ? await getProjectSchema() : await ctx.project.exportSchema(__WEBPACK_EXTERNAL_MODULE__gant_lowcode_lowcode_types__.IPublicEnumTransformStage.Save);
            // 获取 schema，并修正
            const schema = await fixSchema(originalSchema);
            console.log("got schema: ", schema);
            setState((prev)=>({
                    ...prev,
                    schema,
                    originalSchema
                }));
            // 出码...
            const result = await (0, __WEBPACK_EXTERNAL_MODULE__gant_lowcode_code_generator_standalone_loader__.generateCode)({
                solution: "gantLegacy",
                schema,
                flattenResult: true,
                builderOptions: {
                    // mode: 'fully',
                    rootPaths: [
                        'packages',
                        'icost'
                    ],
                    pagePaths: [],
                    configPaths: []
                }
            });
            console.log("generated: ", result);
            setState((prev)=>({
                    ...prev,
                    loading: false,
                    result
                }));
        } catch (e) {
            console.error("failed to run code generator: ", e);
            setState((prev)=>({
                    ...prev,
                    hasError: true,
                    error: e instanceof Error ? e : new Error(`${e?.message || e}`)
                }));
        }
    };
    let drawerChildren = null;
    if (state.hasError) drawerChildren = /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsx)(__WEBPACK_EXTERNAL_MODULE_antd__.Alert, {
        type: "error",
        message: "出错了",
        description: state.error?.message
    });
    if (state.loading) drawerChildren = /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsx)(__WEBPACK_EXTERNAL_MODULE_antd__.Spin, {
        className: "code-gen-plugin-loading",
        spinning: true,
        tip: "正在出码..."
    });
    drawerChildren = /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsx)(__WEBPACK_EXTERNAL_MODULE__code_gen_result_index_js__.CodeGenResult, {
        result: state.result,
        schema: state.schema,
        originalSchema: state.originalSchema
    });
    return /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsxs)(__WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.Fragment, {
        children: [
            /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsx)(__WEBPACK_EXTERNAL_MODULE_antd__.Button, {
                type: "text",
                onClick: handleClick,
                children: /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsx)(__WEBPACK_EXTERNAL_MODULE__ant_design_icons__.CodeOutlined, {})
            }),
            /*#__PURE__*/ (0, __WEBPACK_EXTERNAL_MODULE_react_jsx_runtime__.jsx)(__WEBPACK_EXTERNAL_MODULE_antd__.Drawer, {
                open: state.open,
                title: "出码结果",
                width: "95vw",
                rootClassName: "code-gen-drawer",
                onClose: ()=>{
                    setState((prev)=>({
                            ...prev,
                            open: false
                        }));
                },
                children: drawerChildren
            })
        ]
    });
}
async function fixSchema(schema) {
    return deepClone({
        ...schema,
        componentsMap: schema.componentsMap.filter((c)=>c.package) // 去掉没有 package 的组件
        .map((c)=>{
            // 修正版本号（对于没有有效版本号的组件，默认使用 latest）
            if (!isValidVersion(c.version)) {
                console.warn('[WARN] got invalid version "%o" for component "%s", use "latest" as fallback', c.version, c);
                return {
                    ...c,
                    version: "latest"
                };
            }
            return c;
        })
    });
}
function deepClone(x) {
    try {
        return JSON.parse(JSON.stringify(x));
    } catch (e) {
        throw new Error(`failed to clone schema -- ${e}`);
    }
}
function isValidVersion(version) {
    if (!version) return false;
    // 对于一些明显非法的版本号过滤下
    if ("{{version}}" === version || "null" === version || "undefined" === version) return false;
    // 对于 latest/beta/rc 这样的 tag 版本号要支持下
    if (/^[a-z][a-z0-9]+([a-z0-9-]+)?$/i.test(version)) return true;
    // 最后支持下所有 semver 能识别的版本
    return null !== (0, __WEBPACK_EXTERNAL_MODULE_semver_functions_coerce_js__["default"])(version);
}
export { CodeGenActionBtn, fixSchema };
